@@ -15,56 +15,58 @@
 """Support for explicit client-side field level encryption."""
 
 import contextlib
-import os
-import subprocess
-import uuid
+import enum
+import socket
 import weakref
+from typing import Any, Mapping, Optional, Sequence
 
 try:
     from pymongocrypt.auto_encrypter import AutoEncrypter
-    from pymongocrypt.errors import MongoCryptError
+    from pymongocrypt.errors import MongoCryptError  # noqa: F401
     from pymongocrypt.explicit_encrypter import ExplicitEncrypter
     from pymongocrypt.mongocrypt import MongoCryptOptions
     from pymongocrypt.state_machine import MongoCryptCallback
+
     _HAVE_PYMONGOCRYPT = True
 except ImportError:
     _HAVE_PYMONGOCRYPT = False
     MongoCryptCallback = object
 
 from bson import _dict_to_bson, decode, encode
+from bson.binary import STANDARD, UUID_SUBTYPE, Binary
 from bson.codec_options import CodecOptions
-from bson.binary import (Binary,
-                         STANDARD,
-                         UUID_SUBTYPE)
 from bson.errors import BSONError
-from bson.raw_bson import (DEFAULT_RAW_BSON_OPTIONS,
-                           RawBSONDocument,
-                           _inflate_bson)
+from bson.raw_bson import DEFAULT_RAW_BSON_OPTIONS, RawBSONDocument, _inflate_bson
 from bson.son import SON
-
-from pymongo.errors import (ConfigurationError,
-                            EncryptionError,
-                            InvalidOperation,
-                            ServerSelectionTimeoutError)
+from pymongo import _csot
+from pymongo.cursor import Cursor
+from pymongo.daemon import _spawn_daemon
 from pymongo.encryption_options import AutoEncryptionOpts
+from pymongo.errors import (
+    ConfigurationError,
+    EncryptionError,
+    InvalidOperation,
+    ServerSelectionTimeoutError,
+)
 from pymongo.mongo_client import MongoClient
-from pymongo.pool import _configured_socket, PoolOptions
+from pymongo.network import BLOCKING_IO_ERRORS
+from pymongo.operations import UpdateOne
+from pymongo.pool import PoolOptions, _configured_socket
 from pymongo.read_concern import ReadConcern
+from pymongo.results import BulkWriteResult, DeleteResult
 from pymongo.ssl_support import get_ssl_context
 from pymongo.uri_parser import parse_host
 from pymongo.write_concern import WriteConcern
-from pymongo.daemon import _spawn_daemon
-
 
 _HTTPS_PORT = 443
 _KMS_CONNECT_TIMEOUT = 10  # TODO: CDRIVER-3262 will define this value.
 _MONGOCRYPTD_TIMEOUT_MS = 10000
 
-_DATA_KEY_OPTS = CodecOptions(document_class=SON, uuid_representation=STANDARD)
+
+_DATA_KEY_OPTS: CodecOptions = CodecOptions(document_class=SON, uuid_representation=STANDARD)
 # Use RawBSONDocument codec options to avoid needlessly decoding
 # documents from the key vault.
-_KEY_VAULT_OPTS = CodecOptions(document_class=RawBSONDocument,
-                               uuid_representation=STANDARD)
+_KEY_VAULT_OPTS = CodecOptions(document_class=RawBSONDocument)
 
 
 @contextlib.contextmanager
@@ -80,9 +82,10 @@ def _wrap_encryption_errors():
         raise EncryptionError(exc)
 
 
-class _EncryptionIO(MongoCryptCallback):
+class _EncryptionIO(MongoCryptCallback):  # type: ignore
     def __init__(self, client, key_vault_coll, mongocryptd_client, opts):
         """Internal class to perform I/O on behalf of pymongocrypt."""
+        self.client_ref: Any
         # Use a weak ref to break reference cycle.
         if client is not None:
             self.client_ref = weakref.ref(client)
@@ -90,8 +93,9 @@ class _EncryptionIO(MongoCryptCallback):
             self.client_ref = None
         self.key_vault_coll = key_vault_coll.with_options(
             codec_options=_KEY_VAULT_OPTS,
-            read_concern=ReadConcern(level='majority'),
-            write_concern=WriteConcern(w='majority'))
+            read_concern=ReadConcern(level="majority"),
+            write_concern=WriteConcern(w="majority"),
+        )
         self.mongocryptd_client = mongocryptd_client
         self.opts = opts
         self._spawned = False
@@ -113,25 +117,34 @@ class _EncryptionIO(MongoCryptCallback):
             # Enable strict certificate verification, OCSP, match hostname, and
             # SNI using the system default CA certificates.
             ctx = get_ssl_context(
-                None,   # certfile
-                None,   # passphrase
-                None,   # ca_certs
-                None,   # crlfile
+                None,  # certfile
+                None,  # passphrase
+                None,  # ca_certs
+                None,  # crlfile
                 False,  # allow_invalid_certificates
                 False,  # allow_invalid_hostnames
-                False)  # disable_ocsp_endpoint_check
-        opts = PoolOptions(connect_timeout=_KMS_CONNECT_TIMEOUT,
-                           socket_timeout=_KMS_CONNECT_TIMEOUT,
-                           ssl_context=ctx)
+                False,
+            )  # disable_ocsp_endpoint_check
+        # CSOT: set timeout for socket creation.
+        connect_timeout = max(_csot.clamp_remaining(_KMS_CONNECT_TIMEOUT), 0.001)
+        opts = PoolOptions(
+            connect_timeout=connect_timeout,
+            socket_timeout=connect_timeout,
+            ssl_context=ctx,
+        )
         host, port = parse_host(endpoint, _HTTPS_PORT)
         conn = _configured_socket((host, port), opts)
         try:
             conn.sendall(message)
             while kms_context.bytes_needed > 0:
+                # CSOT: update timeout.
+                conn.settimeout(max(_csot.clamp_remaining(_KMS_CONNECT_TIMEOUT), 0))
                 data = conn.recv(kms_context.bytes_needed)
                 if not data:
-                    raise OSError('KMS connection closed')
+                    raise OSError("KMS connection closed")
                 kms_context.feed(data)
+        except BLOCKING_IO_ERRORS:
+            raise socket.timeout("timed out")
         finally:
             conn.close()
 
@@ -148,8 +161,7 @@ class _EncryptionIO(MongoCryptCallback):
         :Returns:
           The first document from the listCollections command response as BSON.
         """
-        with self.client_ref()[database].list_collections(
-                filter=RawBSONDocument(filter)) as cursor:
+        with self.client_ref()[database].list_collections(filter=RawBSONDocument(filter)) as cursor:
             for doc in cursor:
                 return _dict_to_bson(doc, False, _DATA_KEY_OPTS)
 
@@ -160,7 +172,7 @@ class _EncryptionIO(MongoCryptCallback):
         successfully.
         """
         self._spawned = True
-        args = [self.opts._mongocryptd_spawn_path or 'mongocryptd']
+        args = [self.opts._mongocryptd_spawn_path or "mongocryptd"]
         args.extend(self.opts._mongocryptd_spawn_args)
         _spawn_daemon(args)
 
@@ -181,15 +193,15 @@ class _EncryptionIO(MongoCryptCallback):
         inflated_cmd = _inflate_bson(cmd, DEFAULT_RAW_BSON_OPTIONS)
         try:
             res = self.mongocryptd_client[database].command(
-                inflated_cmd,
-                codec_options=DEFAULT_RAW_BSON_OPTIONS)
+                inflated_cmd, codec_options=DEFAULT_RAW_BSON_OPTIONS
+            )
         except ServerSelectionTimeoutError:
             if self.opts._mongocryptd_bypass_spawn:
                 raise
             self.spawn()
             res = self.mongocryptd_client[database].command(
-                inflated_cmd,
-                codec_options=DEFAULT_RAW_BSON_OPTIONS)
+                inflated_cmd, codec_options=DEFAULT_RAW_BSON_OPTIONS
+            )
         return res.raw
 
     def fetch_keys(self, filter):
@@ -215,12 +227,12 @@ class _EncryptionIO(MongoCryptCallback):
           The _id of the inserted data key document.
         """
         raw_doc = RawBSONDocument(data_key, _KEY_VAULT_OPTS)
-        data_key_id = raw_doc.get('_id')
-        if not isinstance(data_key_id, uuid.UUID):
-            raise TypeError('data_key _id must be a UUID')
+        data_key_id = raw_doc.get("_id")
+        if not isinstance(data_key_id, Binary) or data_key_id.subtype != UUID_SUBTYPE:
+            raise TypeError("data_key _id must be Binary with a UUID subtype")
 
         self.key_vault_coll.insert_one(raw_doc)
-        return Binary(data_key_id.bytes, subtype=UUID_SUBTYPE)
+        return data_key_id
 
     def bson_encode(self, doc):
         """Encode a document to BSON.
@@ -247,11 +259,32 @@ class _EncryptionIO(MongoCryptCallback):
             self.mongocryptd_client = None
 
 
+class RewrapManyDataKeyResult(object):
+    """Result object returned by a :meth:`~ClientEncryption.rewrap_many_data_key` operation.
+
+    .. versionadded:: 4.2
+    """
+
+    def __init__(self, bulk_write_result: Optional[BulkWriteResult] = None) -> None:
+        self._bulk_write_result = bulk_write_result
+
+    @property
+    def bulk_write_result(self) -> Optional[BulkWriteResult]:
+        """The result of the bulk write operation used to update the key vault
+        collection with one or more rewrapped data keys. If
+        :meth:`~ClientEncryption.rewrap_many_data_key` does not find any matching keys to rewrap,
+        no bulk write operation will be executed and this field will be
+        ``None``.
+        """
+        return self._bulk_write_result
+
+
 class _Encrypter(object):
     """Encrypts and decrypts MongoDB commands.
 
     This class is used to support automatic encryption and decryption of
     MongoDB commands."""
+
     def __init__(self, client, opts):
         """Create a _Encrypter for a client.
 
@@ -263,6 +296,11 @@ class _Encrypter(object):
             schema_map = None
         else:
             schema_map = _dict_to_bson(opts._schema_map, False, _DATA_KEY_OPTS)
+
+        if opts._encrypted_fields_map is None:
+            encrypted_fields_map = None
+        else:
+            encrypted_fields_map = _dict_to_bson(opts._encrypted_fields_map, False, _DATA_KEY_OPTS)
         self._bypass_auto_encryption = opts._bypass_auto_encryption
         self._internal_client = None
 
@@ -273,8 +311,7 @@ class _Encrypter(object):
             # Else - limited pool size, use an internal client.
             if encrypter._internal_client is not None:
                 return encrypter._internal_client
-            internal_client = mongo_client._duplicate(
-                minPoolSize=0, auto_encryption_opts=None)
+            internal_client = mongo_client._duplicate(minPoolSize=0, auto_encryption_opts=None)
             encrypter._internal_client = internal_client
             return internal_client
 
@@ -288,43 +325,45 @@ class _Encrypter(object):
         else:
             metadata_client = _get_internal_client(self, client)
 
-        db, coll = opts._key_vault_namespace.split('.', 1)
+        db, coll = opts._key_vault_namespace.split(".", 1)
         key_vault_coll = key_vault_client[db][coll]
 
-        mongocryptd_client = MongoClient(
-            opts._mongocryptd_uri, connect=False,
-            serverSelectionTimeoutMS=_MONGOCRYPTD_TIMEOUT_MS)
+        mongocryptd_client: MongoClient = MongoClient(
+            opts._mongocryptd_uri, connect=False, serverSelectionTimeoutMS=_MONGOCRYPTD_TIMEOUT_MS
+        )
 
-        io_callbacks = _EncryptionIO(
-            metadata_client, key_vault_coll, mongocryptd_client, opts)
-        self._auto_encrypter = AutoEncrypter(io_callbacks, MongoCryptOptions(
-            opts._kms_providers, schema_map))
+        io_callbacks = _EncryptionIO(metadata_client, key_vault_coll, mongocryptd_client, opts)
+        self._auto_encrypter = AutoEncrypter(
+            io_callbacks,
+            MongoCryptOptions(
+                opts._kms_providers,
+                schema_map,
+                crypt_shared_lib_path=opts._crypt_shared_lib_path,
+                crypt_shared_lib_required=opts._crypt_shared_lib_required,
+                bypass_encryption=opts._bypass_auto_encryption,
+                encrypted_fields_map=encrypted_fields_map,
+                bypass_query_analysis=opts._bypass_query_analysis,
+            ),
+        )
         self._closed = False
 
-    def encrypt(self, database, cmd, check_keys, codec_options):
+    def encrypt(self, database, cmd, codec_options):
         """Encrypt a MongoDB command.
 
         :Parameters:
           - `database`: The database for this command.
           - `cmd`: A command document.
-          - `check_keys`: If True, check `cmd` for invalid keys.
           - `codec_options`: The CodecOptions to use while encoding `cmd`.
 
         :Returns:
           The encrypted command to execute.
         """
         self._check_closed()
-        # Workaround for $clusterTime which is incompatible with
-        # check_keys.
-        cluster_time = check_keys and cmd.pop('$clusterTime', None)
-        encoded_cmd = _dict_to_bson(cmd, check_keys, codec_options)
+        encoded_cmd = _dict_to_bson(cmd, False, codec_options)
         with _wrap_encryption_errors():
             encrypted_cmd = self._auto_encrypter.encrypt(database, encoded_cmd)
             # TODO: PYTHON-1922 avoid decoding the encrypted_cmd.
-            encrypt_cmd = _inflate_bson(
-                encrypted_cmd, DEFAULT_RAW_BSON_OPTIONS)
-            if cluster_time:
-                encrypt_cmd['$clusterTime'] = cluster_time
+            encrypt_cmd = _inflate_bson(encrypted_cmd, DEFAULT_RAW_BSON_OPTIONS)
             return encrypt_cmd
 
     def decrypt(self, response):
@@ -353,19 +392,55 @@ class _Encrypter(object):
             self._internal_client = None
 
 
-class Algorithm(object):
+class Algorithm(str, enum.Enum):
     """An enum that defines the supported encryption algorithms."""
-    AEAD_AES_256_CBC_HMAC_SHA_512_Deterministic = (
-        "AEAD_AES_256_CBC_HMAC_SHA_512-Deterministic")
-    AEAD_AES_256_CBC_HMAC_SHA_512_Random = (
-        "AEAD_AES_256_CBC_HMAC_SHA_512-Random")
+
+    AEAD_AES_256_CBC_HMAC_SHA_512_Deterministic = "AEAD_AES_256_CBC_HMAC_SHA_512-Deterministic"
+    """AEAD_AES_256_CBC_HMAC_SHA_512_Deterministic."""
+    AEAD_AES_256_CBC_HMAC_SHA_512_Random = "AEAD_AES_256_CBC_HMAC_SHA_512-Random"
+    """AEAD_AES_256_CBC_HMAC_SHA_512_Random."""
+    INDEXED = "Indexed"
+    """Indexed.
+
+    .. note:: Support for Queryable Encryption is in beta.
+       Backwards-breaking changes may be made before the final release.
+
+    .. versionadded:: 4.2
+    """
+    UNINDEXED = "Unindexed"
+    """Unindexed.
+
+    .. note:: Support for Queryable Encryption is in beta.
+       Backwards-breaking changes may be made before the final release.
+
+    .. versionadded:: 4.2
+    """
+
+
+class QueryType(str, enum.Enum):
+    """**(BETA)** An enum that defines the supported values for explicit encryption query_type.
+
+    .. note:: Support for Queryable Encryption is in beta.
+       Backwards-breaking changes may be made before the final release.
+
+    .. versionadded:: 4.2
+    """
+
+    EQUALITY = "equality"
+    """Used to encrypt a value for an equality query."""
 
 
 class ClientEncryption(object):
     """Explicit client-side field level encryption."""
 
-    def __init__(self, kms_providers, key_vault_namespace, key_vault_client,
-                 codec_options, kms_tls_options=None):
+    def __init__(
+        self,
+        kms_providers: Mapping[str, Any],
+        key_vault_namespace: str,
+        key_vault_client: MongoClient,
+        codec_options: CodecOptions,
+        kms_tls_options: Optional[Mapping[str, Any]] = None,
+    ) -> None:
         """Explicit client-side field level encryption.
 
         The ClientEncryption class encapsulates explicit operations on a key
@@ -439,28 +514,39 @@ class ClientEncryption(object):
             raise ConfigurationError(
                 "client-side field level encryption requires the pymongocrypt "
                 "library: install a compatible version with: "
-                "python -m pip install 'pymongo[encryption]'")
+                "python -m pip install 'pymongo[encryption]'"
+            )
 
         if not isinstance(codec_options, CodecOptions):
-            raise TypeError("codec_options must be an instance of "
-                            "bson.codec_options.CodecOptions")
+            raise TypeError("codec_options must be an instance of bson.codec_options.CodecOptions")
 
         self._kms_providers = kms_providers
         self._key_vault_namespace = key_vault_namespace
         self._key_vault_client = key_vault_client
         self._codec_options = codec_options
 
-        db, coll = key_vault_namespace.split('.', 1)
+        db, coll = key_vault_namespace.split(".", 1)
         key_vault_coll = key_vault_client[db][coll]
 
-        opts = AutoEncryptionOpts(kms_providers, key_vault_namespace,
-                                  kms_tls_options=kms_tls_options)
-        self._io_callbacks = _EncryptionIO(None, key_vault_coll, None, opts)
+        opts = AutoEncryptionOpts(
+            kms_providers, key_vault_namespace, kms_tls_options=kms_tls_options
+        )
+        self._io_callbacks: Optional[_EncryptionIO] = _EncryptionIO(
+            None, key_vault_coll, None, opts
+        )
         self._encryption = ExplicitEncrypter(
-            self._io_callbacks, MongoCryptOptions(kms_providers, None))
+            self._io_callbacks, MongoCryptOptions(kms_providers, None)
+        )
+        # Use the same key vault collection as the callback.
+        self._key_vault_coll = self._io_callbacks.key_vault_coll
 
-    def create_data_key(self, kms_provider, master_key=None,
-                        key_alt_names=None):
+    def create_data_key(
+        self,
+        kms_provider: str,
+        master_key: Optional[Mapping[str, Any]] = None,
+        key_alt_names: Optional[Sequence[str]] = None,
+        key_material: Optional[bytes] = None,
+    ) -> Binary:
         """Create and insert a new data key into the key vault collection.
 
         :Parameters:
@@ -520,19 +606,35 @@ class ClientEncryption(object):
               # reference the key with the alternate name
               client_encryption.encrypt("457-55-5462", keyAltName="name1",
                                         algorithm=Algorithm.AEAD_AES_256_CBC_HMAC_SHA_512_Random)
+          - `key_material` (optional): Sets the custom key material to be used
+            by the data key for encryption and decryption.
 
         :Returns:
           The ``_id`` of the created data key document as a
           :class:`~bson.binary.Binary` with subtype
           :data:`~bson.binary.UUID_SUBTYPE`.
+
+        .. versionchanged:: 4.2
+           Added the `key_material` parameter.
         """
         self._check_closed()
         with _wrap_encryption_errors():
             return self._encryption.create_data_key(
-                kms_provider, master_key=master_key,
-                key_alt_names=key_alt_names)
+                kms_provider,
+                master_key=master_key,
+                key_alt_names=key_alt_names,
+                key_material=key_material,
+            )
 
-    def encrypt(self, value, algorithm, key_id=None, key_alt_name=None):
+    def encrypt(
+        self,
+        value: Any,
+        algorithm: str,
+        key_id: Optional[Binary] = None,
+        key_alt_name: Optional[str] = None,
+        query_type: Optional[str] = None,
+        contention_factor: Optional[int] = None,
+    ) -> Binary:
         """Encrypt a BSON value with a given key and algorithm.
 
         Note that exactly one of ``key_id`` or  ``key_alt_name`` must be
@@ -546,24 +648,43 @@ class ClientEncryption(object):
             :class:`~bson.binary.Binary` with subtype 4 (
             :attr:`~bson.binary.UUID_SUBTYPE`).
           - `key_alt_name`: Identifies a key vault document by 'keyAltName'.
+          - `query_type` (str): **(BETA)** The query type to execute. See
+            :class:`QueryType` for valid options.
+          - `contention_factor` (int): **(BETA)** The contention factor to use
+            when the algorithm is :attr:`Algorithm.INDEXED`.  An integer value
+            *must* be given when the :attr:`Algorithm.INDEXED` algorithm is
+            used.
+
+        .. note:: `query_type` and `contention_factor` are part of the
+           Queryable Encryption beta. Backwards-breaking changes may be made before the
+           final release.
 
         :Returns:
           The encrypted value, a :class:`~bson.binary.Binary` with subtype 6.
+
+        .. versionchanged:: 4.2
+           Added the `query_type` and `contention_factor` parameters.
+
         """
         self._check_closed()
-        if (key_id is not None and not (
-                isinstance(key_id, Binary) and
-                key_id.subtype == UUID_SUBTYPE)):
-            raise TypeError(
-                'key_id must be a bson.binary.Binary with subtype 4')
+        if key_id is not None and not (
+            isinstance(key_id, Binary) and key_id.subtype == UUID_SUBTYPE
+        ):
+            raise TypeError("key_id must be a bson.binary.Binary with subtype 4")
 
-        doc = encode({'v': value}, codec_options=self._codec_options)
+        doc = encode({"v": value}, codec_options=self._codec_options)
         with _wrap_encryption_errors():
             encrypted_doc = self._encryption.encrypt(
-                doc, algorithm, key_id=key_id, key_alt_name=key_alt_name)
-            return decode(encrypted_doc)['v']
+                doc,
+                algorithm,
+                key_id=key_id,
+                key_alt_name=key_alt_name,
+                query_type=query_type,
+                contention_factor=contention_factor,
+            )
+            return decode(encrypted_doc)["v"]  # type: ignore[index]
 
-    def decrypt(self, value):
+    def decrypt(self, value: Binary) -> Any:
         """Decrypt an encrypted value.
 
         :Parameters:
@@ -575,26 +696,177 @@ class ClientEncryption(object):
         """
         self._check_closed()
         if not (isinstance(value, Binary) and value.subtype == 6):
-            raise TypeError(
-                'value to decrypt must be a bson.binary.Binary with subtype 6')
+            raise TypeError("value to decrypt must be a bson.binary.Binary with subtype 6")
 
         with _wrap_encryption_errors():
-            doc = encode({'v': value})
+            doc = encode({"v": value})
             decrypted_doc = self._encryption.decrypt(doc)
-            return decode(decrypted_doc,
-                          codec_options=self._codec_options)['v']
+            return decode(decrypted_doc, codec_options=self._codec_options)["v"]
 
-    def __enter__(self):
+    def get_key(self, id: Binary) -> Optional[RawBSONDocument]:
+        """Get a data key by id.
+
+        :Parameters:
+          - `id` (Binary): The UUID of a key a which must be a
+            :class:`~bson.binary.Binary` with subtype 4 (
+            :attr:`~bson.binary.UUID_SUBTYPE`).
+
+        :Returns:
+          The key document.
+
+        .. versionadded:: 4.2
+        """
+        self._check_closed()
+        return self._key_vault_coll.find_one({"_id": id})
+
+    def get_keys(self) -> Cursor[RawBSONDocument]:
+        """Get all of the data keys.
+
+        :Returns:
+          An instance of :class:`~pymongo.cursor.Cursor` over the data key
+          documents.
+
+        .. versionadded:: 4.2
+        """
+        self._check_closed()
+        return self._key_vault_coll.find({})
+
+    def delete_key(self, id: Binary) -> DeleteResult:
+        """Delete a key document in the key vault collection that has the given ``key_id``.
+
+        :Parameters:
+          - `id` (Binary): The UUID of a key a which must be a
+            :class:`~bson.binary.Binary` with subtype 4 (
+            :attr:`~bson.binary.UUID_SUBTYPE`).
+
+        :Returns:
+          The delete result.
+
+        .. versionadded:: 4.2
+        """
+        self._check_closed()
+        return self._key_vault_coll.delete_one({"_id": id})
+
+    def add_key_alt_name(self, id: Binary, key_alt_name: str) -> Any:
+        """Add ``key_alt_name`` to the set of alternate names in the key document with UUID ``key_id``.
+
+        :Parameters:
+          - ``id``: The UUID of a key a which must be a
+            :class:`~bson.binary.Binary` with subtype 4 (
+            :attr:`~bson.binary.UUID_SUBTYPE`).
+          - ``key_alt_name``: The key alternate name to add.
+
+        :Returns:
+          The previous version of the key document.
+
+        .. versionadded:: 4.2
+        """
+        self._check_closed()
+        update = {"$addToSet": {"keyAltNames": key_alt_name}}
+        return self._key_vault_coll.find_one_and_update({"_id": id}, update)
+
+    def get_key_by_alt_name(self, key_alt_name: str) -> Optional[RawBSONDocument]:
+        """Get a key document in the key vault collection that has the given ``key_alt_name``.
+
+        :Parameters:
+          - `key_alt_name`: (str): The key alternate name of the key to get.
+
+        :Returns:
+          The key document.
+
+        .. versionadded:: 4.2
+        """
+        self._check_closed()
+        return self._key_vault_coll.find_one({"keyAltNames": key_alt_name})
+
+    def remove_key_alt_name(self, id: Binary, key_alt_name: str) -> Optional[RawBSONDocument]:
+        """Remove ``key_alt_name`` from the set of keyAltNames in the key document with UUID ``id``.
+
+        Also removes the ``keyAltNames`` field from the key document if it would otherwise be empty.
+
+        :Parameters:
+          - ``id``: The UUID of a key a which must be a
+            :class:`~bson.binary.Binary` with subtype 4 (
+            :attr:`~bson.binary.UUID_SUBTYPE`).
+          - ``key_alt_name``: The key alternate name to remove.
+
+        :Returns:
+          Returns the previous version of the key document.
+
+        .. versionadded:: 4.2
+        """
+        self._check_closed()
+        pipeline = [
+            {
+                "$set": {
+                    "keyAltNames": {
+                        "$cond": [
+                            {"$eq": ["$keyAltNames", [key_alt_name]]},
+                            "$$REMOVE",
+                            {
+                                "$filter": {
+                                    "input": "$keyAltNames",
+                                    "cond": {"$ne": ["$$this", key_alt_name]},
+                                }
+                            },
+                        ]
+                    }
+                }
+            }
+        ]
+        return self._key_vault_coll.find_one_and_update({"_id": id}, pipeline)
+
+    def rewrap_many_data_key(
+        self,
+        filter: Mapping[str, Any],
+        provider: Optional[str] = None,
+        master_key: Optional[Mapping[str, Any]] = None,
+    ) -> RewrapManyDataKeyResult:
+        """Decrypts and encrypts all matching data keys in the key vault with a possibly new `master_key` value.
+
+        :Parameters:
+          - `filter`: A document used to filter the data keys.
+          - `provider`: The new KMS provider to use to encrypt the data keys,
+            or ``None`` to use the current KMS provider(s).
+          - ``master_key``: The master key fields corresponding to the new KMS
+            provider when ``provider`` is not ``None``.
+
+        :Returns:
+          A :class:`RewrapManyDataKeyResult`.
+
+        .. versionadded:: 4.2
+        """
+        self._check_closed()
+        with _wrap_encryption_errors():
+            raw_result = self._encryption.rewrap_many_data_key(filter, provider, master_key)
+            if raw_result is None:
+                return RewrapManyDataKeyResult()
+
+        raw_doc = RawBSONDocument(raw_result, DEFAULT_RAW_BSON_OPTIONS)
+        replacements = []
+        for key in raw_doc["v"]:
+            update_model = {
+                "$set": {"keyMaterial": key["keyMaterial"], "masterKey": key["masterKey"]},
+                "$currentDate": {"updateDate": True},
+            }
+            op = UpdateOne({"_id": key["_id"]}, update_model)
+            replacements.append(op)
+        if not replacements:
+            return RewrapManyDataKeyResult()
+        result = self._key_vault_coll.bulk_write(replacements)
+        return RewrapManyDataKeyResult(result)
+
+    def __enter__(self) -> "ClientEncryption":
         return self
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
         self.close()
 
     def _check_closed(self):
         if self._encryption is None:
             raise InvalidOperation("Cannot use closed ClientEncryption")
 
-    def close(self):
+    def close(self) -> None:
         """Release resources.
 
         Note that using this class in a with-statement will automatically call
